import pygame
import sys
from random import randint

WIDTH = 800
HEIGHT = 600

BAT_SIZE = 128

ROWS_OF_BRICKS = 7
DEFAUTLT_FPS = 30

BRICK_VERTICAL_OFFSET = 64
TOP_OFFSET = 32
BRICK_PADDING = 2
BRICK_SCORE = 10

BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
BLUE = (0, 0, 255)
GREEN = (0, 255, 0)
RED = (255, 0, 0)
YELLOW = (255, 216, 0)

wall = [[]]


def reset_wall():
    global wall
    wall = [
        [randint(1, 5) for x in range(columns)]
        for y in range(ROWS_OF_BRICKS)]

    print wall


def reset_ball():
    global bat_x
    global ball_vx
    global ball_vy
    global ball_rect
    bat_x = WIDTH / 2 - (BAT_SIZE / 2)
    ball_vx = 0
    ball_vy = 0
    ball_rect.x = bat_x + (BAT_SIZE / 2 - ball_width / 2)
    ball_rect.y = bat_y - ball_height

    
pygame.mixer.init(frequency=22050, size=-16, channels=2, buffer=512)
pygame.init()
clock = pygame.time.Clock()
screen = pygame.display.set_mode([WIDTH, HEIGHT])
screen.fill(BLACK)
font = pygame.font.SysFont("monospace", 15)

fps = DEFAUTLT_FPS

# game
DEFAULT_LIVES = 3
score = 0
lives = DEFAULT_LIVES

# load the ball image
ball = pygame.image.load("ball.png")
ball_rect = ball.get_rect()
ball_width = ball_rect.width
ball_height = ball_rect.height

ball_rect.x = 100
ball_rect.y = 200

# lives ball
lives_ball = pygame.image.load("ball.png")
lives_ball_rect = ball.get_rect()
lives_ball_rect.x = 0
lives_ball_rect.y = 8

# velocity
DEFAULT_VX = 4 #3
DEFAULT_VY = 16
ball_vx = 0
ball_vy = 0

# bat position
bat_x = WIDTH / 2 - (BAT_SIZE / 2)
bat_y = HEIGHT - 16

# load sound...
pygame.mixer.set_num_channels(10)
voice = pygame.mixer.Channel(8)

sound_bat = pygame.mixer.Sound("bat_hit.wav")
sound_brick = pygame.mixer.Sound("brick_hit.wav")

# roughly figure out how many columns will fit on screen
dummy_brick = pygame.image.load("brick_green.png")
brick_width = dummy_brick.get_rect().width
brick_height = dummy_brick.get_rect().height

columns = WIDTH / dummy_brick.get_rect().width
reset_wall()

pygame.mixer.music.load("music.mp3")
pygame.mixer.music.play(-1)

remaining_bricks = ROWS_OF_BRICKS * columns

while 1:
    pygame.event.pump()
    clock.tick(fps) #frames per second

    # capture keyboard input and update the game
    keys = pygame.key.get_pressed()

    if keys[pygame.K_ESCAPE] or keys[pygame.K_q]:
        pygame.quit()
        sys.exit(0)

    if keys[pygame.K_LEFT]:
        bat_x -= 32

    elif keys[pygame.K_RIGHT]:
        bat_x += 32

    elif keys[pygame.K_SPACE]:
        if lives > 0 and ball_vy == 0:
            ball_vx = DEFAULT_VX
            ball_vy = DEFAULT_VY

    elif keys[pygame.K_r]:
        if lives == 0:
            reset_ball()
            lives = DEFAULT_LIVES
            score = 0
            fps = DEFAUTLT_FPS
            remaining_bricks = ROWS_OF_BRICKS * columns
            reset_wall()

    # collision with top
    if ball_vy < 0 and ball_rect.y < TOP_OFFSET :
        ball_vy *= -1

    # off screen
    if ball_vy > 0 and ball_rect.y > HEIGHT + 16:
        lives -= 1
        reset_ball()

    # bat colltion...
    elif ball_vy > 0 and ball_rect.y + ball_height >= bat_y \
            and (ball_rect.x + (ball_width / 2) > bat_x - (ball_width / 2)
                 and ball_rect.x < bat_x + BAT_SIZE + (ball_width / 2)):
        ball_vy *= -1
        bat_middle = bat_x + (BAT_SIZE / 2)

        if ball_rect.x > bat_middle:
            ball_vx += (0.1 * (ball_rect.x - bat_middle))
            ball_vx = min(ball_vx, 8)
        else:
            ball_vx -= (0.1 * (bat_middle - ball_rect.x))
            ball_vx = max(ball_vx, -8)

        sound_bat.play()

    if ball_rect.x + ball_width > WIDTH or ball_rect.x <= 0:
        ball_vx *= -1
        
    # work out if the ball has hit a brick
    ball_col = ((ball_rect.x + (ball_width / 2)) // (brick_width + 2))
    ball_row = ((ball_rect.y - (BRICK_VERTICAL_OFFSET - (ball_width / 2))) // (brick_height + 2))

    block_value = 0

    if (ball_row >= 0 and ball_row <= (ROWS_OF_BRICKS - 1)) \
            and (ball_col >= 0 and ball_col <= (columns - 1)):
        block_value = wall[ball_row][ball_col]
        
    if block_value != 0:
        wall[ball_row][ball_col] = 0
        ball_vy *= -1
        score += BRICK_SCORE
        remaining_bricks -= 1
        sound_brick.play()


    #If the ball has stopped moving, put it on the bat...
    if ball_vx == 0 :
        ball_rect.x = bat_x + (BAT_SIZE / 2 - ball_width / 2)
        ball_rect.y = bat_y - ball_height

    # move the ball normally
    else:
        ball_rect.x = ball_rect.x + ball_vx
        ball_rect.y = ball_rect.y + ball_vy

    # ************** Draw stuff ****************
    # black out the screen.
    screen.fill(BLACK)

    # draw the wall...
    for row in range(len(wall)):
        for col in range(len(wall[row])):
            
            brick = None
            if wall[row][col] != 0:
                val = randint(1,5)
            else:
                val = 0

            if val == 1:
                brick = pygame.image.load("brick_green.png")
            elif val == 2:
                brick = pygame.image.load("brick_blue.png")
            elif val == 3:
                brick = pygame.image.load("brick_yellow.png")
            elif val == 4:
                brick = pygame.image.load("brick_orange.png")
            elif val == 5:
                brick = pygame.image.load("brick_red.png")

            if brick is not None:
                brick_rect = brick.get_rect()

                brick_rect.x = col * (brick_width + BRICK_PADDING)
                brick_rect.y = BRICK_VERTICAL_OFFSET + (row * (brick_height + BRICK_PADDING))

                screen.blit(brick, brick_rect)

    # draw the ball
    screen.blit(ball, ball_rect)
    
    # draw the bat using a simple line
    pygame.draw.line(screen, WHITE, [bat_x, bat_y], [bat_x + BAT_SIZE, HEIGHT - 16], 8)

    # draw the score
    label = font.render("SCORE: " + str(int(score)), 1, WHITE)
    screen.blit(label, (8, 8))

    # draw top edge
    pygame.draw.line(screen, WHITE, [0, TOP_OFFSET], [WIDTH, TOP_OFFSET], 1)

    for l in range(lives):
        lives_ball_rect.x = (WIDTH - 8) - (ball_width * lives) + (l * ball_width)
        screen.blit(lives_ball, lives_ball_rect)

    if lives == 0:
        label = font.render("NO MORE LIVES!!!", 1, RED)
        screen.blit(label, (WIDTH / 2 - (label.get_width() / 2), HEIGHT / 2))

        last_label_y = HEIGHT / 2
        label = font.render("Score - " + str(score), 1, YELLOW)
        screen.blit(label, (WIDTH / 2 - (label.get_width() / 2), last_label_y + label.get_height()))

        last_label_y = last_label_y + label.get_height()        
        label = font.render("(Hit R to restart)", 1, WHITE)
        screen.blit(label, (WIDTH / 2 - (label.get_width() / 2), last_label_y + label.get_height()))

        

    # render everything in the buffer...
    pygame.display.flip()

    if remaining_bricks == 0 :
        reset_ball()
        fps *= 1.10
        remaining_bricks = ROWS_OF_BRICKS * columns 
        reset_wall()
        





        

